==> RaspberryClient-0.1.1-py2.py3-none-any.whl: raspberryclient/raspberryclient.py <==
__version__ = "0.1.1"

import os
import sys
import time
import logging
import requests
import traceback
import subprocess
from logging.handlers import RotatingFileHandler

import pyaudio
import timeout_decorator
from six.moves import queue
from google.cloud import speech
from google.cloud import texttospeech
from google.cloud.speech import enums
from google.cloud.speech import types
from google.api_core.exceptions import DeadlineExceeded

from dbmodel import DbModel as Names


class RaspberryClient:

    RATE = 16000
    CHUNK = int(RATE / 10)
    LANGUAGE_CODE = 'cs-CZ'  # a BCP-47 language tag

    FORMATTER = logging.Formatter('[%(asctime)s] {%(pathname)s:%(lineno)d} %(levelname)s - %(funcName)s -  %(message)s')

    def __init__(self, user_id):
        self.USER_ID = user_id
        self.HOST = sys.argv[1] if len(sys.argv) > 1 else "https://brainquist-checker.develop.promethist.ai"

        self.LOG_FILENAME = "./logs/" + self.USER_ID + "_debug.txt"

        # Instantiates a client
        self.client_TTS = texttospeech.TextToSpeechClient()
        self.client_STT = speech.SpeechClient()

        self.logger = self.init_logger("myLogger")
        self.init_peripheral()

        self.is_interrupted = False
        self.should_check_new_task = True

    class MicrophoneStream(object):
        """Opens a recording stream as a generator yielding the audio chunks."""

        def __init__(self, rate, chunk):
            self._rate = rate
            self._chunk = chunk
            self.all_data = []

            # Create a thread-safe buffer of audio data
            self._buff = queue.Queue()
            self.closed = True

        def __enter__(self):
            self._audio_interface = pyaudio.PyAudio()
            self._audio_stream = self._audio_interface.open(
                format=pyaudio.paInt16,
                # The API currently only supports 1-channel (mono) audio
                # https://goo.gl/z757pE
                channels=1, rate=self._rate,
                input=True, frames_per_buffer=self._chunk,
                # Run the audio stream asynchronously to fill the buffer object.
                # This is necessary so that the input device's buffer doesn't
                # overflow while the calling thread makes network requests, etc.
                stream_callback=self._fill_buffer,
            )

            self.closed = False

            return self

        def __exit__(self, type, value, traceback):
            self._audio_stream.stop_stream()
            self._audio_stream.close()
            self.closed = True
            # Signal the generator to terminate so that the client's
            # streaming_recognize method will not block the process termination.
            self._buff.put(None)
            self._audio_interface.terminate()

        def _fill_buffer(self, in_data, frame_count, time_info, status_flags):
            """Continuously collect data from the audio stream, into the buffer."""
            self._buff.put(in_data)
            return None, pyaudio.paContinue

        def generator(self):
            while not self.closed:
                # Use a blocking get() to ensure there's at least one chunk of
                # data, and stop iteration if the chunk is None, indicating the
                # end of the audio stream.
                chunk = self._buff.get()
                if chunk is None:
                    return
                data = [chunk]

                # Now consume whatever other data's still buffered.
                while True:
                    try:
                        chunk = self._buff.get(block=False)
                        if chunk is None:
                            return
                        data.append(chunk)
                    except queue.Empty:
                        break
                yield b''.join(data)

    @timeout_decorator.timeout(8)
    def listen_print_loop(self, responses):
        best_alternative = "{{ Text nebyl rozpoznán. }}"
        best_confidence_score = 0
        was_silent = True
        for response in responses:
            for result in response.results:
                was_silent = False
                self.logger.info('Finished: {}'.format(result.is_final))
                self.logger.info('Stability: {}'.format(result.stability))
                alternatives = result.alternatives
                # The alternatives are ordered from most likely to least.
                for alternative in alternatives:
                    self.logger.info('Confidence: {}'.format(alternative.confidence))
                    self.logger.info(u'Transcript: {}'.format(alternative.transcript))
                    if alternative.confidence >= best_confidence_score:
                        best_alternative = alternative.transcript
                        best_confidence_score = alternative.confidence
                if result.is_final:
                    return best_alternative, best_confidence_score
            if was_silent:
                return best_alternative, best_confidence_score

    def init_peripheral(self):
        # Raspberry PI
        try:
            from gpiozero import LED, Button

            LED_PIN = 25
            self.led = LED(LED_PIN)
            self.button = Button(23)

        # MacOS
        except ImportError:
            class FakeLED:

                def on(self):
                    print("Led On")

                def off(self):
                    print("Led Off")

            class FakeButton:
                def __init__(self):
                    self.when_pressed = lambda x: print("was clicked")

                def wait_for_press(self):
                    time.sleep(5)

            self.led = FakeLED()
            self.button = FakeButton()

            from pynput import keyboard # has to run as root because of permission

            def on_press(key):
                try:
                    print('alphanumeric key {0} pressed'.format(key.char))
                    self.button.when_pressed()
                except AttributeError:
                    print('special key {0} pressed'.format(key))

            # ...or, in a non-blocking fashion:
            listener = keyboard.Listener(
                on_press=on_press)
            listener.start()

    def get_console_handler(self):
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setFormatter(self.FORMATTER)
        console_handler.setLevel(logging.DEBUG)
        return console_handler

    def get_file_handler(self):
        file_handler = RotatingFileHandler(self.LOG_FILENAME, maxBytes=10.1.100, backupCount=5)
        file_handler.setFormatter(self.FORMATTER)
        file_handler.setLevel(logging.DEBUG)
        return file_handler

    def init_logger(self, logger_name):
        os.makedirs("temp", exist_ok=True)
        os.makedirs("logs", exist_ok=True)
        os.makedirs("logs/audio", exist_ok=True)

        logger = logging.getLogger(logger_name)
        logger.setLevel(logging.DEBUG)  # better to have too much log than not enough
        logger.addHandler(self.get_console_handler())
        logger.addHandler(self.get_file_handler())
        # with this pattern, it's rarely necessary to propagate the error up to parent
        logger.propagate = False
        return logger

    def open_mic_stream(self, streaming_config):
        """
        Will open a mic stream for 8 seconds - prevention to infinite listening
        :param streaming_config:
        :return:
        """
        with self.MicrophoneStream(self.RATE, self.CHUNK) as stream:
            audio_generator = stream.generator()
            _requests = (types.StreamingRecognizeRequest(audio_content=content)
                         for content in audio_generator)

            responses = self.client_STT.streaming_recognize(streaming_config, _requests)
            text, confidence_score = self.listen_print_loop(responses)
            return text, confidence_score

    def stt_google(self, data):
        confidence_score = 0.0

        self.led.on()

        config = types.RecognitionConfig(
            encoding=enums.RecognitionConfig.AudioEncoding.LINEAR16,
            sample_rate_hertz=self.RATE,
            language_code=self.LANGUAGE_CODE,
            max_alternatives=5,
            speech_contexts=[speech.types.SpeechContext(
                phrases=data[Names.EXPECTED_PHRASES] if Names.EXPECTED_PHRASES in data else [])],
            use_enhanced=True
        )
        streaming_config = types.StreamingRecognitionConfig(
            config=config,
            single_utterance=True,
            interim_results=True)
        try:
            text, confidence_score = self.open_mic_stream(streaming_config)
        except timeout_decorator.TimeoutError:
            text = "{{ Text nerozpoznán. }}"
            self.logger.error(traceback.format_exc())
        except DeadlineExceeded:
            text = "{{ Text rozpoznán s chybou. }}"
            self.logger.debug("google.api_core.exceptions.DeadlineExceeded")

        self.led.off()

        self.logger.info(text)
        self.logger.info(confidence_score)

        return text, confidence_score

    def tts_google(self, text, speaking_rate=1.0):
        self.logger.info(text)

        config = texttospeech.types.AudioConfig(
            audio_encoding=texttospeech.enums.AudioEncoding.MP3,
            speaking_rate=speaking_rate,
            pitch=0)

        voice = texttospeech.types.VoiceSelectionParams(
            language_code='cs-CZ',
            name="cs-CZ-Wavenet-A",
            ssml_gender=texttospeech.enums.SsmlVoiceGender.FEMALE)

        synthesis_input = texttospeech.types.SynthesisInput(ssml=text)
        response = self.client_TTS.synthesize_speech(synthesis_input, voice, config)

        src = "./temp/" + self.USER_ID + "_" + str(time.time()) + ".mp3"
        with open(src, "wb") as f:
            # Write the response to the output file.
            f.write(response.audio_content)

        cmd = 'exec mpg321 ' + src
        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
        while p.poll() is None:
            if self.is_interrupted:
                self.logger.debug("is_interrupted")
                p.kill()
            else:
                self.logger.debug("sleeping")
                time.sleep(1)
        # subprocess.call(["mpg321", src])  # waiting to complete
        os.remove(src)

    def logic(self, data):
        self.logger.info(data)
        speaking_rate = data[Names.SPEAKING_RATE] if Names.SPEAKING_RATE in data else 1.0
        self.tts_google(data[Names.BRAINQUIST_TEXT], speaking_rate)

        if self.is_interrupted:
            self.should_check_new_task = True
            self.is_interrupted = False
            self.tts_google("<speak><p><s><break time='200ms'/>Konec</s></p></speak>", 1.0)
            return

        if data[Names.END_CONVERSATION] and data[Names.END_TASK]:
            self.should_check_new_task = True
            self.is_interrupted = False
            return

        if data[Names.NEXT_MODE] == "speak":
            data = requests.post(self.HOST + "/client/routine/" + self.USER_ID + "/start", json=data, timeout=5).json()
            self.logic(data)
        elif data[Names.NEXT_MODE] == "listen":
            data[Names.USER_TEXT], data[Names.CONFIDENCE_SCORE] = self.stt_google(data)
            if data[Names.INITIAL_PHASE]:
                data = requests.post(self.HOST + "/client/start/" + self.USER_ID, json=data, timeout=5).json()
            else:
                data = requests.post(self.HOST + "/client/routine/" + self.USER_ID, json=data, timeout=5).json()
            self.logic(data)

    def force_add_task(self):
        self.logger.debug("force_add_task-POST")
        try:
            self.led.on()
            r = requests.post(self.HOST + "/client/routine/force_add/" + self.USER_ID, timeout=5)
            self.logger.info(r.json())
            self.led.off()
        except Exception as e:
            self.logger.error(e)
            self.led.on()
            time.sleep(0.5)
            self.led.off()
            time.sleep(0.5)
            self.led.on()
            time.sleep(0.5)
            self.led.off()

    def interrupt(self):
        if not self.should_check_new_task:
            self.logger.debug("force_add_task-is_interrupted-True")
            self.is_interrupted = True

    def is_new_version(self):
        if "DEBUG" not in os.environ:
            if requests.get(
                    "https://promethist.ai/repository/pip/brainquist/raspberryclient/version.txt").text.strip() != __version__:
                return True  # should update
        return False

    def check_new_job(self):
        if self.should_check_new_task:
            r = requests.get(self.HOST + "/client/job/" + self.USER_ID, timeout=5)
            self.logger.info(r.json())
            if Names.NO_TASK in r.json():
                pass
            else:
                self.should_check_new_task = False
                self.logic(r.json())

    def run(self):
        self.button.when_pressed = self.interrupt

        # self.tts_google("<speak>Zařízení je připraveno k použití.</speak>")

        if "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ \
                or not os.path.isfile(os.environ["GOOGLE_APPLICATION_CREDENTIALS"]):
            self.logger.error("You should set proper GOOGLE_APPLICATION_CREDENTIALS.")

            if os.path.isfile("/home/pi/cred.json"):
                os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = "/home/pi/cred.json"
            else:
                self.logger.error("Or at least place cred.json into /home/pi/ directory")
                return False

        # if "DEBUG" in os.environ and int(os.environ["DEBUG"]) == 1:
        # self.force_add_task()

        while True:
            try:
                self.button.wait_for_press()
                self.force_add_task()
                self.check_new_job()
                if self.is_new_version():
                    return
            except (SystemExit, Exception, ConnectionRefusedError):
                self.should_check_new_task = True
                tb = traceback.format_exc()
                self.logger.error(tb)
                time.sleep(15)

==> RaspberryClient-0.1.2-py2.py3-none-any.whl: raspberryclient/raspberryclient.py <==
__version__ = "0.1.2"

import os
import sys
import time
import logging
import requests
import traceback
import subprocess
from logging.handlers import RotatingFileHandler

import pyaudio
import timeout_decorator
from six.moves import queue
from google.cloud import speech
from google.cloud import texttospeech
from google.cloud.speech import enums
from google.cloud.speech import types
from google.api_core.exceptions import DeadlineExceeded

from dbmodel import DbModel as Names


class RaspberryClient:

    RATE = 16000
    CHUNK = int(RATE / 10)
    LANGUAGE_CODE = 'cs-CZ'  # a BCP-47 language tag

    FORMATTER = logging.Formatter('[%(asctime)s] {%(pathname)s:%(lineno)d} %(levelname)s - %(funcName)s -  %(message)s')

    def __init__(self, user_id):
        self.USER_ID = user_id
        self.HOST = sys.argv[1] if len(sys.argv) > 1 else "https://brainquist-checker.develop.promethist.ai"

        self.LOG_FILENAME = "./logs/" + self.USER_ID + "_debug.txt"

        # Instantiates a client
        self.client_TTS = texttospeech.TextToSpeechClient()
        self.client_STT = speech.SpeechClient()

        self.logger = self.init_logger("myLogger")
        self.init_peripheral()

        self.is_interrupted = False
        self.should_check_new_task = True

    class MicrophoneStream(object):
        """Opens a recording stream as a generator yielding the audio chunks."""

        def __init__(self, rate, chunk):
            self._rate = rate
            self._chunk = chunk
            self.all_data = []

            # Create a thread-safe buffer of audio data
            self._buff = queue.Queue()
            self.closed = True

        def __enter__(self):
            self._audio_interface = pyaudio.PyAudio()
            self._audio_stream = self._audio_interface.open(
                format=pyaudio.paInt16,
                # The API currently only supports 1-channel (mono) audio
                # https://goo.gl/z757pE
                channels=1, rate=self._rate,
                input=True, frames_per_buffer=self._chunk,
                # Run the audio stream asynchronously to fill the buffer object.
                # This is necessary so that the input device's buffer doesn't
                # overflow while the calling thread makes network requests, etc.
                stream_callback=self._fill_buffer,
            )

            self.closed = False

            return self

        def __exit__(self, type, value, traceback):
            self._audio_stream.stop_stream()
            self._audio_stream.close()
            self.closed = True
            # Signal the generator to terminate so that the client's
            # streaming_recognize method will not block the process termination.
            self._buff.put(None)
            self._audio_interface.terminate()

        def _fill_buffer(self, in_data, frame_count, time_info, status_flags):
            """Continuously collect data from the audio stream, into the buffer."""
            self._buff.put(in_data)
            return None, pyaudio.paContinue

        def generator(self):
            while not self.closed:
                # Use a blocking get() to ensure there's at least one chunk of
                # data, and stop iteration if the chunk is None, indicating the
                # end of the audio stream.
                chunk = self._buff.get()
                if chunk is None:
                    return
                data = [chunk]

                # Now consume whatever other data's still buffered.
                while True:
                    try:
                        chunk = self._buff.get(block=False)
                        if chunk is None:
                            return
                        data.append(chunk)
                    except queue.Empty:
                        break
                yield b''.join(data)

    @timeout_decorator.timeout(8)
    def listen_print_loop(self, responses):
        best_alternative = "{{ Text nebyl rozpoznán. }}"
        best_confidence_score = 0
        was_silent = True
        for response in responses:
            for result in response.results:
                was_silent = False
                self.logger.info('Finished: {}'.format(result.is_final))
                self.logger.info('Stability: {}'.format(result.stability))
                alternatives = result.alternatives
                # The alternatives are ordered from most likely to least.
                for alternative in alternatives:
                    self.logger.info('Confidence: {}'.format(alternative.confidence))
                    self.logger.info(u'Transcript: {}'.format(alternative.transcript))
                    if alternative.confidence >= best_confidence_score:
                        best_alternative = alternative.transcript
                        best_confidence_score = alternative.confidence
                if result.is_final:
                    return best_alternative, best_confidence_score
            if was_silent:
                return best_alternative, best_confidence_score

    def init_peripheral(self):
        # Raspberry PI
        try:
            from gpiozero import LED, Button

            LED_PIN = 25
            self.led = LED(LED_PIN)
            self.button = Button(23)

        # MacOS
        except ImportError:
            class FakeLED:

                def on(self):
                    print("Led On")

                def off(self):
                    print("Led Off")

            class FakeButton:
                def __init__(self):
                    self.when_pressed = lambda x: print("was clicked")

                def wait_for_press(self):
                    time.sleep(5)

            self.led = FakeLED()
            self.button = FakeButton()

            from pynput import keyboard # has to run as root because of permission

            def on_press(key):
                try:
                    print('alphanumeric key {0} pressed'.format(key.char))
                    self.button.when_pressed()
                except AttributeError:
                    print('special key {0} pressed'.format(key))

            # ...or, in a non-blocking fashion:
            listener = keyboard.Listener(
                on_press=on_press)
            listener.start()

    def get_console_handler(self):
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setFormatter(self.FORMATTER)
        console_handler.setLevel(logging.DEBUG)
        return console_handler

    def get_file_handler(self):
        file_handler = RotatingFileHandler(self.LOG_FILENAME, maxBytes=10000000, backupCount=10)
        file_handler.setFormatter(self.FORMATTER)
        file_handler.setLevel(logging.DEBUG)
        return file_handler

    def init_logger(self, logger_name):
        os.makedirs("temp", exist_ok=True)
        os.makedirs("logs", exist_ok=True)
        os.makedirs("logs/audio", exist_ok=True)

        logger = logging.getLogger(logger_name)
        logger.setLevel(logging.DEBUG)  # better to have too much log than not enough
        logger.addHandler(self.get_console_handler())
        logger.addHandler(self.get_file_handler())
        # with this pattern, it's rarely necessary to propagate the error up to parent
        logger.propagate = False
        return logger

    def open_mic_stream(self, streaming_config):
        """
        Will open a mic stream for 8 seconds - prevention to infinite listening
        :param streaming_config:
        :return:
        """
        with self.MicrophoneStream(self.RATE, self.CHUNK) as stream:
            audio_generator = stream.generator()
            _requests = (types.StreamingRecognizeRequest(audio_content=content)
                         for content in audio_generator)

            responses = self.client_STT.streaming_recognize(streaming_config, _requests)
            text, confidence_score = self.listen_print_loop(responses)
            return text, confidence_score

    def stt_google(self, data):
        confidence_score = 0.0

        self.led.on()

        config = types.RecognitionConfig(
            encoding=enums.RecognitionConfig.AudioEncoding.LINEAR16,
            sample_rate_hertz=self.RATE,
            language_code=self.LANGUAGE_CODE,
            max_alternatives=5,
            speech_contexts=[speech.types.SpeechContext(
                phrases=data[Names.EXPECTED_PHRASES] if Names.EXPECTED_PHRASES in data else [])],
            use_enhanced=True
        )
        streaming_config = types.StreamingRecognitionConfig(
            config=config,
            single_utterance=True,
            interim_results=True)
        try:
            text, confidence_score = self.open_mic_stream(streaming_config)
        except timeout_decorator.TimeoutError:
            text = "{{ Text nerozpoznán. }}"
            self.logger.error(traceback.format_exc())
        except DeadlineExceeded:
            text = "{{ Text rozpoznán s chybou. }}"
            self.logger.debug("google.api_core.exceptions.DeadlineExceeded")

        self.led.off()

        self.logger.info(text)
        self.logger.info(confidence_score)

        return text, confidence_score

    def tts_google(self, text, speaking_rate=1.0):
        self.logger.info(text)

        config = texttospeech.types.AudioConfig(
            audio_encoding=texttospeech.enums.AudioEncoding.MP3,
            speaking_rate=speaking_rate,
            pitch=0)

        voice = texttospeech.types.VoiceSelectionParams(
            language_code='cs-CZ',
            name="cs-CZ-Wavenet-A",
            ssml_gender=texttospeech.enums.SsmlVoiceGender.FEMALE)

        synthesis_input = texttospeech.types.SynthesisInput(ssml=text)
        response = self.client_TTS.synthesize_speech(synthesis_input, voice, config)

        src = "./temp/" + self.USER_ID + "_" + str(time.time()) + ".mp3"
        with open(src, "wb") as f:
            # Write the response to the output file.
            f.write(response.audio_content)

        cmd = 'exec mpg321 ' + src
        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
        while p.poll() is None:
            if self.is_interrupted:
                self.logger.debug("is_interrupted")
                p.kill()
            else:
                self.logger.debug("sleeping")
                time.sleep(1)
        # subprocess.call(["mpg321", src])  # waiting to complete
        os.remove(src)

    def logic(self, data):
        self.logger.info(data)
        speaking_rate = data[Names.SPEAKING_RATE] if Names.SPEAKING_RATE in data else 1.0
        self.tts_google(data[Names.BRAINQUIST_TEXT], speaking_rate)

        if self.is_interrupted:
            self.should_check_new_task = True
            self.is_interrupted = False
            self.tts_google("<speak><p><s><break time='200ms'/>Konec</s></p></speak>", 1.0)
            return

        if data[Names.END_CONVERSATION] and data[Names.END_TASK]:
            self.should_check_new_task = True
            self.is_interrupted = False
            return

        if data[Names.NEXT_MODE] == "speak":
            data = requests.post(self.HOST + "/client/routine/" + self.USER_ID + "/start", json=data, timeout=5).json()
            self.logic(data)
        elif data[Names.NEXT_MODE] == "listen":
            data[Names.USER_TEXT], data[Names.CONFIDENCE_SCORE] = self.stt_google(data)
            if data[Names.INITIAL_PHASE]:
                data = requests.post(self.HOST + "/client/start/" + self.USER_ID, json=data, timeout=5).json()
            else:
                data = requests.post(self.HOST + "/client/routine/" + self.USER_ID, json=data, timeout=5).json()
            self.logic(data)

    def force_add_task(self):
        self.logger.debug("force_add_task-POST")
        try:
            self.led.on()
            r = requests.post(self.HOST + "/client/routine/force_add/" + self.USER_ID, timeout=5)
            self.logger.info(r.json())
            self.led.off()
        except Exception as e:
            self.logger.error(e)
            self.led.on()
            time.sleep(0.5)
            self.led.off()
            time.sleep(0.5)
            self.led.on()
            time.sleep(0.5)
            self.led.off()

    def interrupt(self):
        if not self.should_check_new_task:
            self.logger.debug("force_add_task-is_interrupted-True")
            self.is_interrupted = True

    def is_new_version(self):
        if "DEBUG" not in os.environ:
            if requests.get(
                    "https://promethist.ai/repository/pip/brainquist/raspberryclient/version.txt").text.strip() != __version__:
                return True  # should update
        return False

    def check_new_job(self):
        if self.should_check_new_task:
            r = requests.get(self.HOST + "/client/job/" + self.USER_ID, timeout=5)
            self.logger.info(r.json())
            if Names.NO_TASK in r.json():
                pass
            else:
                self.should_check_new_task = False
                self.logic(r.json())

    def run(self):
        self.button.when_pressed = self.interrupt

        # self.tts_google("<speak>Zařízení je připraveno k použití.</speak>")

        if "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ \
                or not os.path.isfile(os.environ["GOOGLE_APPLICATION_CREDENTIALS"]):
            self.logger.error("You should set proper GOOGLE_APPLICATION_CREDENTIALS.")

            if os.path.isfile("/home/pi/cred.json"):
                os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = "/home/pi/cred.json"
            else:
                self.logger.error("Or at least place cred.json into /home/pi/ directory")
                return False

        # if "DEBUG" in os.environ and int(os.environ["DEBUG"]) == 1:
        # self.force_add_task()

        while True:
            try:
                self.button.wait_for_press()
                self.force_add_task()
                self.check_new_job()
                if self.is_new_version():
                    return
            except (SystemExit, Exception, ConnectionRefusedError):
                self.should_check_new_task = True
                tb = traceback.format_exc()
                self.logger.error(tb)
                time.sleep(15)
